@@ -1,5 +1,5 @@
 '''
-SimpleEval - (C) 2013/2014 Daniel Fairhead
+SimpleEval - (C) 2013/2015 Daniel Fairhead
 -------------------------------------
 
 An short, easy to use, safe and reasonably extensible expression evaluator.
@@ -35,7 +35,10 @@ modifications and many improvments.
 
 -------------------------------------
 Contributors:
-corro (Robin Baumgartner) (py3k)
+- corro (Robin Baumgartner) (py3k)
+- dratchkov (David R) (nested dicts)
+- marky1991 (Mark Young) (slicing)
+- T045T (Nils Berg) (!=, py3kstr, obj.attributes)
 
 -------------------------------------
 Usage:
@@ -76,6 +79,7 @@ well:
 '''
 
 import ast
+import sys
 import operator as op
 from random import random
 
@@ -84,6 +88,7 @@ from random import random
 
 MAX_STRING_LENGTH = 100000
 MAX_POWER = 4000000 # highest exponent
+PYTHON3 = sys.version_info[0] == 3
 
 ########################################
 # Exceptions:
@@ -114,6 +119,14 @@ class NameNotDefined(InvalidExpression):
         # pylint: disable=bad-super-call
         super(InvalidExpression, self).__init__(self.message)
 
+class AttributeDoesNotExist(InvalidExpression):
+    '''attribute does not exist'''
+    def __init__(self, attr, expression):
+        self.message = "Attribute '{0}' does not exist in expression '{1}'".format(
+            attr, expression)
+        self.attr = attr
+        self.expression = expression
+ 
 class FeatureNotAvailable(InvalidExpression):
     ''' What you're trying to do is not allowed. '''
     pass
@@ -165,12 +178,12 @@ def safe_add(a, b): # pylint: disable=invalid-name
 
 DEFAULT_OPERATORS = {ast.Add: safe_add, ast.Sub: op.sub, ast.Mult: safe_mult,
                      ast.Div: op.truediv, ast.Pow: safe_power, ast.Mod: op.mod,
-                     ast.Eq: op.eq, ast.Gt: op.gt, ast.Lt: op.lt,
+                     ast.Eq: op.eq, ast.NotEq: op.ne, ast.Gt: op.gt, ast.Lt: op.lt,
                      ast.GtE: op.ge, ast.LtE: op.le, ast.USub: op.neg,
                      ast.UAdd: op.pos}
 
 DEFAULT_FUNCTIONS = {"rand": random, "randint": random_int,
-                     "int": int, "float": float, "str": unicode}
+                     "int": int, "float": float, "str": str if PYTHON3 else unicode}
 
 DEFAULT_NAMES = {"True": True, "False": False}
 
@@ -262,7 +275,13 @@ class SimpleEval(object): # pylint: disable=too-few-public-methods
 
         elif isinstance(node, ast.Name): # a, b, c...
             try:
-                if isinstance(self.names, dict):
+                #This happens at least for slicing
+                #This is a safe thing to do because it is impossible
+                #that there is a true exression assigning to none
+                #(the compiler rejects it, so you can't even pass that to ast.parse)
+                if node.id == "None":
+                    return None
+                elif isinstance(self.names, dict):
                     return self.names[node.id]
                 elif callable(self.names):
                     return self.names(node)
@@ -275,8 +294,34 @@ class SimpleEval(object): # pylint: disable=too-few-public-methods
                 raise NameNotDefined(node.id, self.expr)
 
         elif isinstance(node, ast.Subscript): # b[1]
-            return self._eval(node.value)[self._eval(node.slice.value)]
+            return self._eval(node.value)[self._eval(node.slice)]
 
+        elif isinstance(node, ast.Attribute): # a.b.c
+            try:
+                return self._eval(node.value)[node.attr]
+            except (KeyError, TypeError):
+                pass
+
+            # Maybe the base object is an actual object, not just a dict
+            try:
+                return getattr(self._eval(node.value), node.attr)
+            except (AttributeError, TypeError):
+                pass
+
+            # If it is neither, raise an exception
+            raise AttributeDoesNotExist(node.attr, self.expr)
+
+        elif isinstance(node, ast.Index):
+            return self._eval(node.value)
+        elif isinstance(node, ast.Slice):
+            lower = upper = step = None
+            if node.lower is not None:
+                lower = self._eval(node.lower)
+            if node.upper is not None:
+                upper = self._eval(node.upper)
+            if node.step is not None:
+                step = self._eval(node.step)
+            return slice(lower, upper, step)
         else:
             raise FeatureNotAvailable("Sorry, {0} is not available in this "
                                       "evaluator".format(type(node).__name__ ))
