@@ -7,9 +7,9 @@
 '''
 # pylint: disable=too-many-public-methods, missing-docstring
 
-import unittest
+import unittest, operator, ast
 import simpleeval
-from simpleeval import SimpleEval, NameNotDefined, InvalidExpression, simple_eval
+from simpleeval import SimpleEval, NameNotDefined, InvalidExpression, AttributeDoesNotExist, simple_eval
 
 class DRYTest(unittest.TestCase):
     ''' Stuff we need to do every test, let's do here instead..
@@ -43,6 +43,7 @@ class TestBasic(DRYTest):
         self.t('1 - 1 or 21', True)
         self.t('1 - 1 and 11', False)
         self.t('110 == 100 + 10 and True', True)
+        self.t('110 != 100 + 10 and True', False)
 
     def test_maths_with_floats(self):
         self.t("11.02 - 9.1", 1.92)
@@ -84,7 +85,24 @@ class TestBasic(DRYTest):
 
         self.t('int("20") + int(0.22*100)', 42)
         self.t('float("42")', 42.0)
-        self.t('"Test Stuff!" + str(11)', u"Test Stuff!11")
+        self.t('"Test Stuff!" + str(11)', "Test Stuff!11")
+
+    def test_slicing(self):
+        self.s.operators[ast.Slice] = operator.getslice if hasattr(operator, "getslice") else operator.getitem
+        self.t("'hello'[1]", "e")
+        self.t("'hello'[:]", "hello")
+        self.t("'hello'[:3]", "hel")
+        self.t("'hello'[3:]", "lo")
+        self.t("'hello'[::2]", "hlo")
+        self.t("'hello'[::-1]", "olleh")
+        self.t("'hello'[3::]", "lo")
+        self.t("'hello'[:3:]", "hel")
+        self.t("'hello'[1:3]", "el")
+        self.t("'hello'[1:3:]", "el")
+        self.t("'hello'[1::2]", "el")
+        self.t("'hello'[:1:2]", "h")
+        self.t("'hello'[1:3:1]", "el")
+        self.t("'hello'[1:3:2]", "e")
 
 class TestFunctions(DRYTest):
     ''' Functions for expressions to play with '''
@@ -106,7 +124,7 @@ class TestFunctions(DRYTest):
 
         # simple load:
 
-        self.s.functions = {u"read": load_file}
+        self.s.functions = {"read": load_file}
         self.t("read('file.txt')", "42")
 
         # and we should have *replaced* the default functions. Let's check:
@@ -226,6 +244,11 @@ class TestNames(DRYTest):
         with self.assertRaises(InvalidExpression):
             self.t('s', 21)
 
+        self.s.names = {'a' : {'b': {'c': 42}}}
+
+        with self.assertRaises(AttributeDoesNotExist):
+            self.t('a.b.d**2', 42)
+
 
     def test_dict(self):
         ''' using a normal dict for names lookup '''
@@ -277,6 +300,39 @@ class TestNames(DRYTest):
 
         self.assertEqual(self.s.names['c']['c']['c'], 11)
 
+        # nested dict
+
+        self.s.names = {'a' : {'b': {'c': 42}}}
+
+        self.t("a.b.c*2", 84)
+
+        self.t("a.b.c = 11", 11)
+
+        self.assertEqual(self.s.names['a']['b']['c'], 42)
+
+        self.t("a.d = 11", 11)
+
+        with self.assertRaises(KeyError):
+            self.assertEqual(self.s.names['a']['d'], 11)
+
+    def test_object(self):
+        ''' using an object for name lookp '''
+        class TestObject(object):
+            pass
+        o = TestObject()
+        o.a = 23
+        o.b = 42
+        o.c = TestObject()
+        o.c.d = 9001
+
+        self.s.names = {'o' : o}
+
+        self.t('o', o)
+        self.t('o.a', 23)
+        self.t('o.b + o.c.d', 9043)
+        with self.assertRaises(AttributeDoesNotExist):
+            self.t('o.d', None)
+
     def test_func(self):
         ''' using a function for 'names lookup' '''
 
@@ -319,3 +375,6 @@ class Test_simple_eval(unittest.TestCase):
     def test_default_functions(self):
         self.assertEqual(simple_eval('rand() < 1.0 and rand() > -0.01'), True)
         self.assertEqual(simple_eval('randint(200) < 200 and rand() > 0'), True)
+
+if __name__ == '__main__':
+    unittest.main()
